@@ -3,6 +3,7 @@
 $Id: arrayprint.py,v 1.9 2005/09/13 13:58:44 teoliphant Exp $
 """
 __all__ = ["array2string", "set_printoptions", "get_printoptions"]
+__docformat__ = 'restructuredtext'
 
 #
 # Written by Konrad Hinsen <hinsenk@ere.umontreal.ca>
@@ -68,21 +69,21 @@ def set_printoptions(precision=None, threshold=None, edgeitems=None,
                      linewidth=None, suppress=None):
     """Set options associated with printing.
 
-    precision  the default number of digits of precision for floating
-                   point output
-                   (default 8)
-    threshold  total number of array elements which trigger summarization
-                   rather than full repr.
-                   (default 1000)
-    edgeitems  number of array items in summary at beginning and end of
-                   each dimension.
-                   (default 3)
-    linewidth  the number of characters per line for the purpose of inserting
-                   line breaks.
-                   (default 75)
-    suppress    Boolean value indicating whether or not suppress printing
-                   of small floating point values using scientific notation
-                   (default False)
+    :Parameters:
+        precision : int
+            Number of digits of precision for floating point output (default 8).
+        threshold : int
+            Total number of array elements which trigger summarization
+            rather than full repr (default 1000).
+        edgeitems : int
+            Number of array items in summary at beginning and end of
+            each dimension (default 3).
+        linewidth : int
+            The number of characters per line for the purpose of inserting
+            line breaks (default 75).
+        suppress : bool
+            Whether or not suppress printing of small floating point values
+            using scientific notation (default False).
     """
 
     global _summaryThreshold, _summaryEdgeItems, _float_output_precision, \
@@ -100,6 +101,19 @@ def set_printoptions(precision=None, threshold=None, edgeitems=None,
     return
 
 def get_printoptions():
+    """Return the current print options.
+
+    :Returns:
+        precision : int
+        threshold : int
+        edgeitems : int
+        linewidth : int
+        suppress : bool
+
+    :SeeAlso:
+     - set_printoptions : parameter descriptions
+
+    """
     return _float_output_precision, _summaryThreshold, _summaryEdgeItems, \
            _line_width, _float_output_suppress_small
 
@@ -122,6 +136,11 @@ def _leading_trailing(a):
         b = _gen.concatenate(tuple(l))
     return b
 
+def _boolFormatter(x):
+    if x: return ' True'
+    else: return 'False'
+
+
 def _array2string(a, max_line_width, precision, suppress_small, separator=' ',
                   prefix=""):
 
@@ -144,23 +163,23 @@ def _array2string(a, max_line_width, precision, suppress_small, separator=' ',
     try:
         format_function = a._format
     except AttributeError:
-        dtype = a.dtype.type
-        if issubclass(dtype, _nt.bool):
-            format = "%s"
-            format_function = lambda x: format % x
-        if issubclass(dtype, _nt.integer):
+        dtypeobj = a.dtype.type
+        if issubclass(dtypeobj, _nt.bool_):
+            # make sure True and False line up.
+            format_function = _boolFormatter
+        elif issubclass(dtypeobj, _nt.integer):
             max_str_len = max(len(str(max_reduce(data))),
                               len(str(min_reduce(data))))
             format = '%' + str(max_str_len) + 'd'
             format_function = lambda x: _formatInteger(x, format)
-        elif issubclass(dtype, _nt.floating):
-            if issubclass(dtype, _nt.longfloat):
+        elif issubclass(dtypeobj, _nt.floating):
+            if issubclass(dtypeobj, _nt.longfloat):
                 format_function = _longfloatFormatter(precision)
             else:
                 format = _floatFormat(data, precision, suppress_small)
                 format_function = lambda x: _formatFloat(x, format)
-        elif issubclass(dtype, _nt.complexfloating):
-            if issubclass(dtype, _nt.clongfloat):
+        elif issubclass(dtypeobj, _nt.complexfloating):
+            if issubclass(dtypeobj, _nt.clongfloat):
                 format_function = _clongfloatFormatter(precision)
             else:
                 real_format = _floatFormat(
@@ -169,18 +188,17 @@ def _array2string(a, max_line_width, precision, suppress_small, separator=' ',
                     data.imag, precision, suppress_small, sign=1)
                 format_function = lambda x: \
                               _formatComplex(x, real_format, imag_format)
-        elif issubclass(dtype, _nt.unicode_) or \
-             issubclass(dtype, _nt.string_):
+        elif issubclass(dtypeobj, _nt.unicode_) or \
+             issubclass(dtypeobj, _nt.string_):
             format = "%s"
             format_function = lambda x: repr(x)
         else:
             format = '%s'
-            format_function = lambda x: format % str(x)
+            format_function = lambda x: str(x)
 
     next_line_prefix = " " # skip over "["
     next_line_prefix += " "*len(prefix)                  # skip over array(
 
-
     lst = _formatArray(a, format_function, len(a.shape), max_line_width,
                        next_line_prefix, separator,
                        _summaryEdgeItems, summary_insert)[:-1]
@@ -201,6 +219,37 @@ def _convert_arrays(obj):
 def array2string(a, max_line_width = None, precision = None,
                  suppress_small = None, separator=' ', prefix="",
                  style=repr):
+    """Return a string representation of an array.
+
+    :Parameters:
+        a : ndarray
+            Input array.
+        max_line_width : int
+            The maximum number of columns the string should span. Newline
+            characters splits the string appropriately after array elements.
+        precision : int
+            Floating point precision.
+        suppress_small : bool
+            Represent very small numbers as zero.
+        separator : string
+            Inserted between elements.
+        prefix : string
+            An array is typically printed as
+
+            'prefix(' + array2string(a) + ')'
+
+            The length of the prefix string is used to align the
+            output correctly.
+        style : function
+
+    Examples
+    --------
+
+    >>> x = N.array([1e-16,1,2,3])
+    >>> print array2string(x,precision=2,separator=',',suppress_small=True)
+    [ 0., 1., 2., 3.]
+
+    """
 
     if a.shape == ():
         x = a.item()
@@ -248,7 +297,6 @@ def _formatArray(a, format_function, rank, max_line_len,
         leading_items, trailing_items, summary_insert1 = 0, len(a), ""
 
     if rank == 1:
-
         s = ""
         line = next_line_prefix
         for i in xrange(leading_items):
@@ -296,6 +344,7 @@ def _formatArray(a, format_function, rank, max_line_len,
 
 def _floatFormat(data, precision, suppress_small, sign = 0):
     exp_format = 0
+    errstate = _gen.seterr(all='ignore')
     non_zero = _uf.absolute(data.compress(_uf.not_equal(data, 0)))
     ##non_zero = _numeric_compress(data) ##
     if len(non_zero) == 0:
@@ -309,6 +358,7 @@ def _floatFormat(data, precision, suppress_small, sign = 0):
         if not suppress_small and (min_val < 0.0001
                                    or max_val/min_val > 1000.):
             exp_format = 1
+    _gen.seterr(**errstate)
     if exp_format:
         large_exponent = 0 < min_val < 1e-99 or max_val >= 1e100
         max_str_len = 8 + precision + large_exponent
