@@ -38,7 +38,7 @@ class chararray(ndarray):
         return self
 
     def __array_finalize__(self, obj):
-        # The b is a special case because it is used for reconstructing. 
+        # The b is a special case because it is used for reconstructing.
         if not _globalvar and self.dtype.char not in 'SUb':
             raise ValueError, "Can only create a chararray from string data."
 
@@ -47,28 +47,28 @@ class chararray(ndarray):
         if isinstance(val, (string_, unicode_)):
             temp = val.rstrip()
             if len(temp) == 0:
-                val = val[0]
+                val = ''
             else:
                 val = temp
         return val
 
-    def __eq__(self, other): 
+    def __eq__(self, other):
         return compare_chararrays(self, other, '==', True)
 
-    def __ne__(self, other): 
+    def __ne__(self, other):
         return compare_chararrays(self, other, '!=', True)
 
     def __ge__(self, other):
-        return compare_chararrays(self, other, '>=', True)        
+        return compare_chararrays(self, other, '>=', True)
 
     def __le__(self, other):
-        return compare_chararrays(self, other, '<=', True)                
+        return compare_chararrays(self, other, '<=', True)
 
     def __gt__(self, other):
         return compare_chararrays(self, other, '>', True)
 
     def __lt__(self, other):
-        return compare_chararrays(self, other, '<', True)        
+        return compare_chararrays(self, other, '<', True)
 
     def __add__(self, other):
         b = broadcast(self, other)
@@ -129,6 +129,9 @@ class chararray(ndarray):
     def __rmod__(self, other):
         return NotImplemented
 
+    def argsort(self, axis=-1, kind='quicksort', order=None):
+        return self.__array__().argsort(axis, kind, order)
+
     def _generalmethod(self, name, myiter):
         res = [None]*myiter.size
         maxsize = -1
