@@ -12,7 +12,7 @@ __all__ = ['newaxis', 'ndarray', 'flatiter', 'ufunc',
            'array_repr', 'array_str', 'set_string_function',
            'little_endian', 'require',
            'fromiter', 'array_equal', 'array_equiv',
-           'indices', 'fromfunction',
+           'indices', 'fromfunction', 'loadtxt', 'savetxt',
            'load', 'loads', 'isscalar', 'binary_repr', 'base_repr',
            'ones', 'identity', 'allclose', 'compare_chararrays', 'putmask',
            'seterr', 'geterr', 'setbufsize', 'getbufsize',
@@ -159,7 +159,7 @@ def require(a, dtype=None, requirements=None):
         subok = 0
     else:
         subok = 1
-        
+
     arr = array(a, dtype=dtype, copy=False, subok=subok)
 
     copychar = 'A'
@@ -176,7 +176,7 @@ def require(a, dtype=None, requirements=None):
         if not arr.flags[prop]:
             arr = arr.copy(copychar)
             break
-    return arr   
+    return arr
 
 def isfortran(a):
     """Returns True if 'a' is arranged in Fortran-order in memory with a.ndim > 1
@@ -219,8 +219,11 @@ def convolve(a,v,mode='full'):
     """Returns the discrete, linear convolution of 1-D sequences a and v; mode
     can be 'valid', 'same', or 'full' to specify size of the resulting sequence.
     """
+    a,v = array(a,ndmin=1),array(v,ndmin=1)
     if (len(v) > len(a)):
         a, v = v, a
+    assert len(a) > 0, 'a cannot be empty'
+    assert len(v) > 0, 'v cannot be empty'
     mode = _mode_from_name(mode)
     return multiarray.correlate(a,asarray(v)[::-1],mode)
 
@@ -240,7 +243,7 @@ def outer(a,b):
 def vdot(a, b):
     """Returns the dot product of 2 vectors (or anything that can be made into
     a vector).
-    
+
     Note: this is not the same as `dot`, as it takes the conjugate of its first
     argument if complex and always returns a scalar."""
     return dot(asarray(a).ravel().conj(), asarray(b).ravel())
@@ -265,7 +268,7 @@ def tensordot(a, b, axes=2):
     the axes to be summed over are given by the axes argument.
     the first element of the sequence determines the axis or axes
     in arr1 to sum over, and the second element in axes argument sequence
-    determines the axis or axes in arr2 to sum over. 
+    determines the axis or axes in arr2 to sum over.
 
     When there is more than one axis to sum over, the corresponding
     arguments to axes should be sequences of the same length with the first
@@ -273,7 +276,7 @@ def tensordot(a, b, axes=2):
     and so forth.
 
     If the axes argument is an integer, N, then the last N dimensions of a
-    and first N dimensions of b are summed over. 
+    and first N dimensions of b are summed over.
     """
     try:
         iter(axes)
@@ -375,8 +378,8 @@ def rollaxis(a, axis, start=0):
         raise ValueError, msg % ('axis', axis, n)
     if not (0 <= start < n+1):
         raise ValueError, msg % ('start', start, n+1)
-    if (axis < start): # it's been removed 
-        start -= 1    
+    if (axis < start): # it's been removed
+        start -= 1
     if axis==start:
         return a
     axes = range(0,n)
@@ -475,28 +478,36 @@ set_string_function(array_repr, 1)
 
 little_endian = (sys.byteorder == 'little')
 
+
 def indices(dimensions, dtype=int):
     """Returns an array representing a grid of indices with row-only, and
     column-only variation.
     """
-    tmp = ones(dimensions, dtype)
-    lst = []
-    for i in range(len(dimensions)):
-        lst.append( add.accumulate(tmp, i, dtype)-1 )
-    return array(lst)
+    dimensions = tuple(dimensions)
+    N = len(dimensions)
+    if N == 0:
+        return array([],dtype=dtype)
+    res = empty((N,)+dimensions, dtype=dtype)
+    for i, dim in enumerate(dimensions):
+        tmp = arange(dim,dtype=dtype)
+        tmp.shape = (1,)*i + (dim,)+(1,)*(N-i-1)
+        newdim = dimensions[:i] + (1,)+ dimensions[i+1:]
+        val = zeros(newdim, dtype)
+        add(tmp, val, res[i])
+    return res
 
 def fromfunction(function, shape, **kwargs):
     """Returns an array constructed by calling a function on a tuple of number
     grids.
-    
-    The function should accept as many arguments as the length of shape and work
-    on array inputs.  The shape argument is a sequence of numbers indicating the
-    length of the desired output for each axis.
 
-    The function can also accept keyword arguments (except dtype), which will be
-    passed through fromfunction to the function itself.  The dtype argument
+    The function should accept as many arguments as the length of shape and
+    work on array inputs.  The shape argument is a sequence of numbers
+    indicating the length of the desired output for each axis.
+
+    The function can also accept keyword arguments (except dtype), which will
+    be passed through fromfunction to the function itself.  The dtype argument
     (default float) determines the data-type of the index grid passed to the
-    function. 
+    function.
     """
     dtype = kwargs.pop('dtype', float)
     args = indices(shape, dtype=dtype)
@@ -532,32 +543,47 @@ _lkup = {
     'C':'1100',
     'D':'1101',
     'E':'1110',
-    'F':'1111',    
+    'F':'1111',
     'L':''}
 
-def binary_repr(num):
+def binary_repr(num, width=None):
     """Return the binary representation of the input number as a string.
 
     This is equivalent to using base_repr with base 2, but about 25x
     faster.
+
+    For negative numbers, if width is not given, a - sign is added to the
+    front. If width is given, the two's complement of the number is
+    returned, with respect to that width.
     """
-    ostr = hex(num)
-    bin = ''
-    for ch in ostr[2:]:
-        bin += _lkup[ch]
-    if '1' in bin:
-        ind = 0
-        while bin[ind] == '0':
-            ind += 1
-        return bin[ind:]
-    else:
+    sign = ''
+    if num < 0:
+        if width is None:
+            sign = '-'
+            num = -num
+        else:
+            # replace num with its 2-complement
+            num = 2**width + num
+    elif num == 0:
         return '0'
+    ostr = hex(num)
+    bin = ''.join([_lkup[ch] for ch in ostr[2:]])
+    bin = bin.lstrip('0')
+    if width is not None:
+        bin = bin.zfill(width)
+    return sign + bin
 
 def base_repr (number, base=2, padding=0):
-    """Return the representation of a number in any given base.
-    """
-    chars = '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ'
+    """Return the representation of a number in the given base.
 
+    Base can't be larger than 36.
+    """
+    if number < 0:
+        raise ValueError("negative numbers not handled in base_repr")
+    if base > 36:
+        raise ValueError("bases greater than 36 not handled in base_repr")
+
+    chars = '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ'
     import math
     lnb = math.log(base)
     res = padding*chars[0]
@@ -584,6 +610,177 @@ def load(file):
         file = _file(file,"rb")
     return _cload(file)
 
+# Adapted from matplotlib
+
+def _getconv(dtype):
+    typ = dtype.type
+    if issubclass(typ, bool_):
+        return lambda x: bool(int(x))
+    if issubclass(typ, integer):
+        return int
+    elif issubclass(typ, floating):
+        return float
+    elif issubclass(typ, complex):
+        return complex
+    else:
+        return str
+
+
+def _string_like(obj):
+    try: obj + ''
+    except (TypeError, ValueError): return 0
+    return 1
+
+def loadtxt(fname, dtype=float, comments='#', delimiter=None, converters=None,
+            skiprows=0, usecols=None, unpack=False):
+    """
+    Load ASCII data from fname into an array and return the array.
+
+    The data must be regular, same number of values in every row
+
+    fname can be a filename or a file handle.  Support for gzipped files is
+    automatic, if the filename ends in .gz
+
+    See scipy.loadmat to read and write matfiles.
+
+    Example usage:
+
+      X = loadtxt('test.dat')  # data in two columns
+      t = X[:,0]
+      y = X[:,1]
+
+    Alternatively, you can do the same with "unpack"; see below
+
+      X = loadtxt('test.dat')    # a matrix of data
+      x = loadtxt('test.dat')    # a single column of data
+
+
+    dtype - the data-type of the resulting array.  If this is a
+    record data-type, the the resulting array will be 1-d and each row will
+    be interpreted as an element of the array. The number of columns
+    used must match the number of fields in the data-type in this case. 
+    
+    comments - the character used to indicate the start of a comment
+    in the file
+
+    delimiter is a string-like character used to seperate values in the
+    file. If delimiter is unspecified or none, any whitespace string is
+    a separator.
+
+    converters, if not None, is a dictionary mapping column number to
+    a function that will convert that column to a float.  Eg, if
+    column 0 is a date string: converters={0:datestr2num}
+
+    skiprows is the number of rows from the top to skip
+
+    usecols, if not None, is a sequence of integer column indexes to
+    extract where 0 is the first column, eg usecols=(1,4,5) to extract
+    just the 2nd, 5th and 6th columns
+
+    unpack, if True, will transpose the matrix allowing you to unpack
+    into named arguments on the left hand side
+
+        t,y = load('test.dat', unpack=True) # for  two column data
+        x,y,z = load('somefile.dat', usecols=(3,5,7), unpack=True)
+
+    """
+
+    if _string_like(fname):
+        if fname.endswith('.gz'):
+            import gzip
+            fh = gzip.open(fname)
+        else:
+            fh = file(fname)
+    elif hasattr(fname, 'seek'):
+        fh = fname
+    else:
+        raise ValueError('fname must be a string or file handle')
+    X = []
+
+    dtype = multiarray.dtype(dtype)
+    defconv = _getconv(dtype)
+    converterseq = None    
+    if converters is None:
+        converters = {}
+        if dtype.names is not None:
+            converterseq = [_getconv(dtype.fields[name][0]) \
+                            for name in dtype.names]
+            
+    for i,line in enumerate(fh):
+        if i<skiprows: continue
+        line = line[:line.find(comments)].strip()
+        if not len(line): continue
+        vals = line.split(delimiter)
+        if converterseq is None:
+           converterseq = [converters.get(j,defconv) \
+                           for j in xrange(len(vals))]
+        if usecols is not None:
+            row = [converterseq[j](vals[j]) for j in usecols]
+        else:
+            row = [converterseq[j](val) for j,val in enumerate(vals)]
+        if dtype.names is not None:
+            row = tuple(row)
+        X.append(row)
+
+    X = array(X, dtype)
+    r,c = X.shape
+    if r==1 or c==1:
+        X.shape = max([r,c]),
+    if unpack: return X.T
+    else:  return X
+
+
+# adjust so that fmt can change across columns if desired. 
+
+def savetxt(fname, X, fmt='%.18e',delimiter=' '):
+    """
+    Save the data in X to file fname using fmt string to convert the
+    data to strings
+
+    fname can be a filename or a file handle.  If the filename ends in .gz,
+    the file is automatically saved in compressed gzip format.  The load()
+    command understands gzipped files transparently.
+
+    Example usage:
+
+    save('test.out', X)         # X is an array
+    save('test1.out', (x,y,z))  # x,y,z equal sized 1D arrays
+    save('test2.out', x)        # x is 1D
+    save('test3.out', x, fmt='%1.4e')  # use exponential notation
+
+    delimiter is used to separate the fields, eg delimiter ',' for
+    comma-separated values
+    """
+
+    if _string_like(fname):
+        if fname.endswith('.gz'):
+            import gzip
+            fh = gzip.open(fname,'wb')
+        else:
+            fh = file(fname,'w')
+    elif hasattr(fname, 'seek'):
+        fh = fname
+    else:
+        raise ValueError('fname must be a string or file handle')
+
+
+    X = asarray(X)
+    origShape = None
+    if len(X.shape)==1:
+        origShape = X.shape
+        X.shape = len(X), 1
+    for row in X:
+        fh.write(delimiter.join([fmt%val for val in row]) + '\n')
+
+    if origShape is not None:
+        X.shape = origShape
+
+
+    
+
+
+
+
 # These are all essentially abbreviations
 # These might wind up in a special abbreviations module
 
@@ -621,10 +818,10 @@ def identity(n, dtype=None):
     a = array([1]+n*[0],dtype=dtype)
     b = empty((n,n),dtype=dtype)
 
-    # Note that this assignment depends on the convention that since the a array
-    # is shorter than the flattened b array, then the a array will be repeated
-    # until it is the appropriate size. Given a's construction, this nicely sets
-    # the diagonal to all ones.
+    # Note that this assignment depends on the convention that since the a
+    # array is shorter than the flattened b array, then the a array will
+    # be repeated until it is the appropriate size. Given a's construction,
+    # this nicely sets the diagonal to all ones.
     b.flat = a
     return b
 
@@ -638,9 +835,21 @@ def allclose(a, b, rtol=1.e-5, atol=1.e-8):
     """
     x = array(a, copy=False)
     y = array(b, copy=False)
-    d = less_equal(absolute(x-y), atol + rtol * absolute(y))
-    return d.ravel().all()
-
+    d1 = less_equal(absolute(x-y), atol + rtol * absolute(y))
+    xinf = isinf(x)
+    yinf = isinf(y)
+    if (not xinf.any() and not yinf.any()):
+        return d1.all()
+    d3 = (x[xinf] == y[yinf])
+    d4 = (~xinf & ~yinf)
+    if d3.size < 2:
+        if d3.size==0:
+            return False
+        return d3
+    if d3.all():
+        return d1[d4].all() 
+    else:
+        return False
 
 def array_equal(a1, a2):
     try:
@@ -688,17 +897,19 @@ def seterr(all=None, divide=None, over=None, under=None, invalid=None):
 
     Example:
 
-    >>> seterr(over='raise')
+    >>> seterr(over='raise') # doctest: +SKIP
     {'over': 'ignore', 'divide': 'ignore', 'invalid': 'ignore', 'under': 'ignore'}
-    >>> seterr(all='warn', over='raise')
+    
+    >>> seterr(all='warn', over='raise') # doctest: +SKIP
     {'over': 'raise', 'divide': 'ignore', 'invalid': 'ignore', 'under': 'ignore'}
-    >>> int16(32000) * int16(3)
+    
+    >>> int16(32000) * int16(3) # doctest: +SKIP
     Traceback (most recent call last):
           File "<stdin>", line 1, in ?
     FloatingPointError: overflow encountered in short_scalars
-    >>> seterr(all='ignore')
+    >>> seterr(all='ignore') # doctest: +SKIP
     {'over': 'ignore', 'divide': 'ignore', 'invalid': 'ignore', 'under': 'ignore'}
-    
+
     """
 
     pyvals = umath.geterrobj()
@@ -785,16 +996,16 @@ def geterrcall():
     """
     return umath.geterrobj()[2]
 
-class _unspecified(object): 
+class _unspecified(object):
     pass
 _Unspecified = _unspecified()
 
 class errstate(object):
     """with errstate(**state): --> operations in following block use given state.
-    
+
     # Set error handling to known state.
-    >>> _ = seterr(invalid='raise', divide='raise', over='raise', under='ignore') 
-    
+    >>> _ = seterr(invalid='raise', divide='raise', over='raise', under='ignore')
+
     |>> a = -arange(3)
     |>> with errstate(invalid='ignore'):
     ...     print sqrt(a)
@@ -812,7 +1023,7 @@ class errstate(object):
     Traceback (most recent call last):
         ...
     FloatingPointError: divide by zero encountered in divide
-    
+
     """
     # Note that we don't want to run the above doctests because they will fail
     # without a from __future__ import with_statement
