@@ -264,7 +264,7 @@ array_getfield(PyArrayObject *self, PyObject *args, PyObject *kwds)
 					 PyArray_DescrConverter,
 					 &dtype, &offset)) return NULL;
 
-	return _ARET(PyArray_GetField(self, dtype, offset));
+	return PyArray_GetField(self, dtype, offset);
 }
 
 
@@ -479,7 +479,9 @@ array_toscalar(PyArrayObject *self, PyObject *args) {
                         PyErr_SetString(PyExc_ValueError, "invalid integer");
                         return NULL;
                 }
-                if (value >= PyArray_SIZE(self)) {
+                factor = PyArray_SIZE(self);
+                if (value < 0) value += factor;
+                if ((value >= factor) || (value < 0)) {
                         PyErr_SetString(PyExc_ValueError, 
                                         "index out of bounds");
                         return NULL;
@@ -615,7 +617,7 @@ array_setscalar(PyArrayObject *self, PyObject *args) {
         else {
                 intp loc, index[MAX_DIMS];
                 PyObject *tupargs;
-                tupargs = PyTuple_GetSlice(args, 1, n+1);
+                tupargs = PyTuple_GetSlice(args, 0, n);
                 nd = PyArray_IntpFromSequence(tupargs, index, MAX_DIMS);
                 Py_DECREF(tupargs);
                 if (nd < n) return NULL;
@@ -651,14 +653,19 @@ array_cast(PyArrayObject *self, PyObject *args)
 			      &descr)) return NULL;
 
 	if (descr == self->descr) {
-		obj = _ARET(PyArray_NewCopy(self,0));
+		obj = _ARET(PyArray_NewCopy(self,NPY_ANYORDER));
 		Py_XDECREF(descr);
 		return obj;
 	}
 	if (descr->names != NULL) {
-		return PyArray_FromArray(self, descr, NPY_FORCECAST);
+                int flags;
+                flags = NPY_FORCECAST;
+                if (PyArray_ISFORTRAN(self)) {
+                        flags |= NPY_FORTRAN;
+                }
+		return PyArray_FromArray(self, descr, flags);
 	}
-	return PyArray_CastToType(self, descr, 0);
+	return PyArray_CastToType(self, descr, PyArray_ISFORTRAN(self));
 }
 
 /* default sub-type implementation */
@@ -865,14 +872,18 @@ array_sort(PyArrayObject *self, PyObject *args, PyObject *kwds)
         if (order == Py_None) order = NULL;
         if (order != NULL) {
                 PyObject *new_name;
+                PyObject *_numpy_internal;
                 saved = self->descr;
                 if (saved->names == NULL) {
                         PyErr_SetString(PyExc_ValueError, "Cannot specify " \
                                         "order when the array has no fields.");
                         return NULL;
                 }
+                _numpy_internal = PyImport_ImportModule("numpy.core._internal");
+                if (_numpy_internal == NULL) return NULL;
                 new_name = PyObject_CallMethod(_numpy_internal, "_newnames",
                                                "OO", saved, order);
+                Py_DECREF(_numpy_internal);
                 if (new_name == NULL) return NULL;
                 newd = PyArray_DescrNew(saved);
                 newd->names = new_name;
@@ -907,14 +918,18 @@ array_argsort(PyArrayObject *self, PyObject *args, PyObject *kwds)
         if (order == Py_None) order = NULL;
         if (order != NULL) {
                 PyObject *new_name;
+                PyObject *_numpy_internal;
                 saved = self->descr;
                 if (saved->names == NULL) {
                         PyErr_SetString(PyExc_ValueError, "Cannot specify " \
                                         "order when the array has no fields.");
                         return NULL;
                 }
+                _numpy_internal = PyImport_ImportModule("numpy.core._internal");
+                if (_numpy_internal == NULL) return NULL;
                 new_name = PyObject_CallMethod(_numpy_internal, "_newnames",
                                                "OO", saved, order);
+                Py_DECREF(_numpy_internal);
                 if (new_name == NULL) return NULL;
                 newd = PyArray_DescrNew(saved);
                 newd->names = new_name;
