@@ -581,6 +581,13 @@ class test_regression(NumpyTestCase):
                   N.rec.fromarrays([(1,2),(3,4)])]:
             assert(a.dtype in [dt0,dt1])
 
+    def check_random_shuffle(self, level=rlevel):
+        """Ticket #374"""
+        a = N.arange(5).reshape((5,1))
+        b = a.copy()
+        N.random.shuffle(b)
+        assert_equal(N.sort(b, axis=0),a)
+
     def check_refcount_vectorize(self, level=rlevel):
         """Ticket #378"""
         def p(x,y): return 123
@@ -595,5 +602,90 @@ class test_regression(NumpyTestCase):
         ca = N.char.array(['Hi','There'])
         assert_equal(ca.startswith('H'),[True,False])
 
+    def check_noncommutative_reduce_accumulate(self, level=rlevel):
+        """Ticket #413"""
+        tosubtract = N.arange(5)
+        todivide = N.array([2.0, 0.5, 0.25])
+        assert_equal(N.subtract.reduce(tosubtract), -10)
+        assert_equal(N.divide.reduce(todivide), 16.0)
+        assert_array_equal(N.subtract.accumulate(tosubtract),
+            N.array([0, -1, -3, -6, -10]))
+        assert_array_equal(N.divide.accumulate(todivide),
+            N.array([2., 4., 16.]))
+
+    def check_mem_polymul(self, level=rlevel):
+        """Ticket #448"""
+        N.polymul([],[1.])
+
+    def check_convolve_empty(self, level=rlevel):
+        """Convolve should raise an error for empty input array."""
+        self.failUnlessRaises(AssertionError,N.convolve,[],[1])
+        self.failUnlessRaises(AssertionError,N.convolve,[1],[])
+
+    def check_multidim_byteswap(self, level=rlevel):
+        """Ticket #449"""
+        r=N.array([(1,(0,1,2))], dtype="i2,3i2")
+        assert_array_equal(r.byteswap(),
+                           N.array([(256,(0,256,512))],r.dtype))
+
+    def check_string_NULL(self, level=rlevel):
+        """Changeset 3557"""
+        assert_equal(N.array("a\x00\x0b\x0c\x00").item(),
+                     'a\x00\x0b\x0c')
+
+    def check_mem_string_concat(self, level=rlevel):
+        """Ticket #469"""
+        x = N.array([])
+        N.append(x,'asdasd\tasdasd')
+
+    def check_matrix_multiply_by_1d_vector(self, level=rlevel) :
+        """Ticket #473"""
+        def mul() :
+            N.mat(N.eye(2))*N.ones(2)
+
+        self.failUnlessRaises(ValueError,mul)
+
+    def check_junk_in_string_fields_of_recarray(self, level=rlevel):
+        """Ticket #483"""
+        r = N.array([['abc']], dtype=[('var1', '|S20')])
+        assert str(r['var1'][0][0]) == 'abc'
+
+    def check_take_output(self, level=rlevel):
+        """Ensure that 'take' honours output parameter."""
+        x = N.arange(12).reshape((3,4))
+        a = N.take(x,[0,2],axis=1)
+        b = N.zeros_like(a)
+        N.take(x,[0,2],axis=1,out=b)
+        assert_array_equal(a,b)
+
+    def check_array_str_64bit(self, level=rlevel):
+        """Ticket #501"""
+        s = N.array([1, N.nan],dtype=N.float64)
+        errstate = N.seterr(all='raise')
+        try:
+            sstr = N.array_str(s)
+        finally:
+            N.seterr(**errstate)
+
+    def check_frompyfunc_endian(self, level=rlevel):
+        """Ticket #503"""
+        from math import radians
+        uradians = N.frompyfunc(radians, 1, 1)
+        big_endian = N.array([83.4, 83.5], dtype='>f8')
+        little_endian = N.array([83.4, 83.5], dtype='<f8')
+        assert_almost_equal(uradians(big_endian).astype(float),
+                            uradians(little_endian).astype(float))
+
+    def check_mem_string_arr(self, level=rlevel):
+        """Ticket #514"""
+        s = "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa"
+        t = []
+        N.hstack((t, s ))
+
+    def check_arr_transpose(self, level=rlevel):
+        """Ticket #516"""
+        x = N.random.rand(*(2,)*16)
+        y = x.transpose(range(16))
+
 if __name__ == "__main__":
     NumpyTest().run()
